import sha
import sys, os, getopt

ROOT = sys.path[0]

# set the location of config.cfg.
try :
	optlist, args = getopt.getopt(sys.argv[1:], "shar", \
		["status", "add", "remove", "quota=", "message_size_limit=", \
		"document_size_limit=", "www=", "filter=", \
		"block_address=", "remote_server=", "user=", "password=", \
		"config-file="])
except :
	print "ERROR : Bad argument.!!"
	sys.exit(1)
else :
	optlist_dict = dict(filter(lambda i : i, filter(lambda i: i[1], optlist)))

	if optlist_dict.has_key("--config-file") :
		cfg_file = optlist_dict["--config-file"]
	elif os.getenv("CFG_FILE") :
		cfg_file = os.getenv("CFG_FILE")
	else :
		cfg_file = "../etc/engine.cfg"

if not os.path.exists(cfg_file) :
	print "ERROR : Configuration file '%s' does not exists. Check it." % cfg_file
	sys.exit(1)

sys.path.append("%s/module" % ROOT)
sys.path.append("%s/resource" % ROOT)

import Config

# read configuration file, config.cfg
CC = Config.Config(cfg_file)
Config_CFG = CC.get_all()

#-------------------------------------------------- default
default = {}

# interval
default["interval"] = 10 # second, default 10 seconds.
default["interval_remote_message"] = 300 # second, default 5 minutes.
default["interval_orphan_clean"] = 3600 # second, default 1 hour.

default["postmaster"] = "Mesian-PostMaster"

if not Config_CFG["Language"].has_key("encoding") :
	default["encoding"] = "euc-kr"
else :
	default["encoding"] = Config_CFG["Language"]["encoding"]

default["quota"] = {}
default["quota"]["default_quota"] = 50000000 # byte
default["quota"]["over_quota"] = 30 # %

# Don't touch this unless you know what to do.
default["item"] = {}
default["item"]["inbox"] = 1
default["item"]["sent"] = 2
default["item"]["draft"] = 3
default["item"]["tmp"] = 4
default["item"]["trash"] = 5
default["item"]["spam"] = 6

default["item"]["default"] = 4

#---------------------------------------------- send message
if not Config_CFG.has_key("SMTP") :
	raise ValueError, "%s must be set in config file." % "SMTP"

if not "method"	in Config_CFG["SMTP"].keys() :
	raise ValueError, "%s must be set in config file." % "Method"

if not "host"	in Config_CFG["SMTP"].keys() :
	raise ValueError, "%s must be set in config file." % "Host"

if not "port"	in Config_CFG["SMTP"].keys() :
	raise ValueError, "%s must be set in config file." % "Port"

default["send"] = Config_CFG["SMTP"]["method"]

smtp = {}
smtp["server"] = Config_CFG["SMTP"]["host"]
smtp["port"] = Config_CFG["SMTP"]["port"]

#---------------------------------------------- IMAP server
if Config_CFG.has_key("IMAP") :
	if "port" in Config_CFG["IMAP"].keys() :
		default["imap_port"] = int(Config_CFG["IMAP"]["port"])

#-------------------------------------------------- db
db = {}

if not Config_CFG.has_key("Database") :
	raise ValueError, "Check the engine/config.cfg"
	sys.exit(1)
else :
	if not "host" in Config_CFG["Database"].keys() :
		raise ValueError, "%s must be set in config file." % "Host"

	if (not "unix_socket" in Config_CFG["Database"].keys()) and \
			(not "port" in Config_CFG["Database"].keys()) :
		raise ValueError, "%s must be set in config file." % "Unix_socket or Port"

	if not "db" in Config_CFG["Database"].keys() :
		raise ValueError, "%s must be set in config file." % "Db"

	if not "user" in Config_CFG["Database"].keys() :
		raise ValueError, "%s must be set in config file." % "User"

	if not "passwd" in Config_CFG["Database"].keys() :
		raise ValueError, "%s must be set in config file." % "Passwd"

	db["host"]			= Config_CFG["Database"]["host"]
	db["db"]				= Config_CFG["Database"]["db"]
	db["user"]			= Config_CFG["Database"]["user"]
	db["passwd"]		= Config_CFG["Database"]["passwd"]

	if "unix_socket" in Config_CFG["Database"].keys() :
		db["unix_socket"]	= Config_CFG["Database"]["unix_socket"]
	else :
		db["port"]			= Config_CFG["Database"]["port"]

#-------------------------------------------------- directory
mail_server = {}
mail_server["root"] = Config_CFG["Directory"]["root"]

directory = {}

if Config_CFG["Directory"].has_key("engine") :
	directory["root"] = Config_CFG["Directory"]["engine"]
else :
	directory["root"] = "%s/engine" % Config_CFG["Directory"]["root"]

if Config_CFG["Directory"].has_key("log") :
	directory["log"] = Config_CFG["Directory"]["log"]
else :
	directory["log"] = "%s/log" % Config_CFG["Directory"]["root"]

directory["pid"] = directory["root"]

www_data = {}
if Config_CFG["Directory"].has_key("www-data") :
	www_data["root"] = Config_CFG["Directory"]["www-data"]
else :
	www_data["root"] = "%s/www-data" % Config_CFG["Directory"]["root"]

qmail = {}
qmail["directory"] = {}
qmail["file"] = {}

qmail["directory"]["root"] = "%s/server" % mail_server["root"]

if Config_CFG["Directory"].has_key("domains") :
	qmail["directory"]["domains"] = Config_CFG["Directory"]["domains"]
else :
	qmail["directory"]["domains"] = "%s/domains" % mail_server["root"]

if Config_CFG["Directory"].has_key("message") :
	qmail["directory"]["message"] = Config_CFG["Directory"]["message"]
else :
	qmail["directory"]["message"] = "%s/message" % mail_server["root"]

qmail["directory"]["bin"] = "%s/bin" % qmail["directory"]["root"]
qmail["directory"]["control"] = "%s/control" % qmail["directory"]["root"]
qmail["directory"]["users"] = "%s/users" % qmail["directory"]["root"]
qmail["directory"]["message_tmp"] = "%s/tmp" % qmail["directory"]["message"]

qmail["file"]["rcpthosts"] = "%s/morercpthosts" % qmail["directory"]["control"]
qmail["file"]["virtualdomains"] = "%s/virtualdomains" % qmail["directory"]["control"]
qmail["file"]["assign"] = "%s/assign" % qmail["directory"]["users"]
qmail["file"]["qmail-newu"] = "%s/qmail-newu" % qmail["directory"]["bin"]
qmail["file"]["qmail-newmrh"] = "%s/qmail-newmrh" % qmail["directory"]["bin"]
qmail["file"]["qmail_default"] = "%s/.qmail-default" % qmail["directory"]["domains"]

#-------------------------------------------------- logging
log = {}
log["type"] = "1"
log["maxbyte"] = 1000000 # 1MB
log["backupcount"] = 7

if Config_CFG.has_key("Log") :
	log["type"] = Config_CFG["Log"]["type"]

	if Config_CFG["Log"]["type"] == "1" and Config_CFG["Log"]["maxbyte"] :
		if Config_CFG["Log"]["maxbyte"] > 100 :
			log["maxbyte"] = int(Config_CFG["Log"]["maxbyte"])

	if Config_CFG["Log"]["type"] == "1" and Config_CFG["Log"]["backupcount"] :
		if Config_CFG["Log"]["backupcount"] > 1 :
			log["backupcount"] = int(Config_CFG["Log"]["backupcount"])

#-------------------------------------------------- anti-virus
file = {}
file["scanner"] = None

if Config_CFG.has_key("Anti-Virus") :

	if Config_CFG["Anti-Virus"]["enable"] == "yes" :
		if Config_CFG["Anti-Virus"]["scanner"] :
			file["scanner"] = Config_CFG["Anti-Virus"]["scanner"]

__author__ =  "Spike^ekipS <spike@spikeekips.net>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "config.py"

