import sys, time, getopt, os
import config

import myMySQLdb, Domain, sql, message

#---------------------------------------------- argument handling

# 	id
# 	name
# 	quota
# 	valid
# 	createdTime
# 	message_size_limit
# 	document_size_limit
# 	filter
# 	user_limit
# 	password
# 	password_plain

USAGE = \
"""
Mesian 0.5 : %(prog_name)s,
   make new domain and remove the registerd domain.

Usage : %(prog_name)s <domain name>
  Ex) %(prog_name)s -a --password="this is password" mydomain.org

   -s, --status             : show domain configuration status.
   -a, --add                : Add new domain
   -r, --remove             : Remove new domain
   --quota                  : quota (M).     > default 70(M)
   --message_size_limit     : message size limit (M).
                                             > default 5(M)
   --document_size_limit    : document size limit (M).
                                             > default 5(M)
   --filter                 : the total number of filter.
                                             > default 10
   --block_address          : the total number of
                              address to block.     > default 7
   --remote_server          : the total number of
                              remote server to fetch
                              message.       > default 7
   --user                   : the total number of users.
                                             > default 5
   --password               : password
   --www                    : www domain name
   --config-file            : use another config.cfg

------------------------------------------------
Bug reports: http://mesian-devel.spikeekips.net
"""

MSG_STATUS = \
"""
Mesian 0.5 : %(prog_name)s

This is the status of registered domains. %(current_time)s
=========================================

* %(location_rcpthosts)s :
%(list_rcpthosts)s

* %(location_virtualdomains)s :
%(list_virtualdomains)s

* %(location_assign)s :
%(list_assign)s

* DATABASE 'domain_priv' table :
%(list_domain_priv)s

------------------------------------------------
Bug reports: http://mesian-devel.spikeekips.net
"""


MSG_CONFIGURATION = \
	"""Your Configuration is,
--------------------------------------------
          Domain Name : %s
               Action : %s
                Quota : %5s
   Message Size Limit : %5s
  Document Size Limit : %5s
        Filter number : %5s
 Block Address number : %5s
 Remote server number : %5s
          User number : %5s
             Password : %s
      WWW Domain Name : %s
--------------------------------------------
	"""
MSG_SUCCESS_REMOVE = """Successfully removed."""
MSG_SUCCESS_ADD = """Successfully added.

====================================
Please restart your QMAIL server.
====================================
"""

ERROR_NOT_MB = \
"""
ERROR : '%s' must be integer value.
        Don't append 'MB', 'KB', 'K' or 'M'
"""

# simple function, This print USAGE of this script.
def print_usage () :
	print USAGE % {"prog_name" : os.path.basename(sys.argv[0])}

if len(sys.argv) < 2 :
	print_usage()
	sys.exit(1)

# parse arguments, parameters
try :
	optlist, args = getopt.getopt(sys.argv[1:], "shar", \
		["status", "add", "remove", "quota=", "message_size_limit=", \
		"document_size_limit=", "www=", "filter=", \
		"block_address=", "remote_server=", "user=", "password=", \
		"config-file="])
except :
	print_usage()
	sys.exit(1)

options = {}
options["quota"] = 70
options["document_size_limit"] = 5
options["message_size_limit"] = 5
options["filter"] = 10
options["block_address"] = 7
options["remote_server"] = 3
options["user"] = 5
www_default_prefix = "mesian"

action = None
show_status = False

for i in optlist :
	if i[0] in ("--add", "-a") :
		action = "add"
	elif i[0] in ("--remove", "-r") :
		action = "remove"

	# option
	elif i[0] == "--quota" :
		try :
			a = int(i[1])
		except :
			print ERROR_NOT_MB % "--quota"
			sys.exit(1)
		else :
			options["quota"] = i[1]

	elif i[0] == "--message_size_limit" :
		try :
			a = int(i[1])
		except :
			print ERROR_NOT_MB % "--message_size_limit"
			sys.exit(1)
		else :
			options["message_size_limit"] = i[1]

	elif i[0] == "--document_size_limit" :
		try :
			a = int(i[1])
		except :
			print ERROR_NOT_MB % "--document_size_limit"
			sys.exit(1)
		else :
			options["document_size_limit"] = i[1]

	elif i[0] == "--filter" :
		options["filter"] = i[1]

	elif i[0] == "--block_address" :
		options["block_address"] = i[1]

	elif i[0] == "--remote_server" :
		options["remote_server"] = i[1]

	elif i[0] == "--user" :
		options["user"] = i[1]

	elif i[0] == "--password" :
		options["password"] = i[1]

	elif i[0] == "--www" :
		options["www"] = i[1]

	elif i[0] in ("--status", "-s") :
		show_status = True

# print status.
if show_status :
	DB = myMySQLdb.myMySQLdb()

	try :
		DB.connect(config.db)
	except ValueError :
		print "** Can not connect DB."
		sys.exit(1)

	DM = Domain.Domain()
	DM.set_db(DB)

	info = DM.status()
	info["prog_name"] = os.path.basename(sys.argv[0])
	info["current_time"] = time.asctime(time.localtime())

	print MSG_STATUS % info

	DB.quit()
	sys.exit(0)

if not action :
	print "Add or Remove? What?"
	print_usage()
	sys.exit(1)

# value check
if action == "add" and \
		("password" not in options.keys() or options["password"].strip() == "") :
	print "(***) YOu must insert password!"
	print_usage()
	sys.exit(1)

domain = sys.argv[-1]

if not message.is_valid_domain(domain) :
	print "(***) \"%s\" is not valid domain name. Try agian." % domain
	print_usage()
	sys.exit(1)

if "www" not in options.keys() or options["www"].strip() == "" :
	www_domain_name = "%s.%s" % (www_default_prefix, domain)
else :
	www_domain_name = options["www"]

if action == "add" :
	print MSG_CONFIGURATION % \
		(domain, action, \
		options["quota"], options["message_size_limit"], \
		options["document_size_limit"], options["filter"], \
		options["block_address"], options["remote_server"], \
		options["user"], \
		options["password"], www_domain_name)

#----------------------------- db connection
DB = myMySQLdb.myMySQLdb()

try :
	DB.connect(config.db)
except ValueError :
	print "** Can not connect DB."
	sys.exit(1)

DM = Domain.Domain()

if action == "add" :
	retval = DM.add(domain)
elif action == "remove" :
	retval = DM.remove(domain)

if retval == False :
	DB.quit()

	print "(***) Failed to %s %s" % (action, domain)
	sys.exit(1)
elif retval == -3 :
	DB.quit()
	print "(***) It's not a vaild domain name, %s" % (domain)
	sys.exit(1)

elif action == "add" and retval == -2 :
	DB.quit()
	print "(***) It already exists, %s" % (domain)
	sys.exit(1)

elif action == "remove" and retval == -2 :
	DB.quit()
	print "(***) It does not exists, %s" % (domain)
	sys.exit(1)

else :
	# add domain info to domain_priv.
	DM.set_db(DB)
	DB.start_transaction(True)

	options["www_domain_name"] = www_domain_name

	try :
		retval = DM.do_DB(action, domain, **options)
	except :
		retval = False

	if retval and action == "remove" :
		print MSG_SUCCESS_REMOVE
	elif retval and action == "add" :
		print MSG_SUCCESS_ADD
	else :
		DB.retval = False

		if action == "add" :
			DM.remove(domain)
		elif action == "remove" :
			DM.add(domain)

		print "(***) Failed to %s %s." % (action, domain)

	DB.close()
	DB.quit()

sys.exit(0)

__author__ =  "Spike^ekipS <spike@spikeekips.net>"
__file__ = "make_domain.py"

