# -*- coding: utf-8 -*-
#import twisted.protocols.imap4 as imap4
import imap4
from twisted.python import components
from twisted.internet.defer import maybeDeferred
import twisted.internet.protocol

import sha, sys

import config, sql, message, MailBox, Privilege

#------------------------------------------------------------
class Factory (twisted.internet.protocol.Factory) :

	def __init__ (self, DB) :
		self.DB = DB

	def buildProtocol(self, addr):
		p = self.protocol()
		p.set_db(self.DB)
		p.factory = self

		return p

class Server (imap4.IMAP4Server) :

	IDENT = 'Twisted IMAP4rev1 for Mesian Ready'

	def __init__ (self, contextFactory = None) :

		#twisted.protocols.imap4.IMAP4Server.__init__(self, contextFactory)
		imap4.IMAP4Server.__init__(self, contextFactory)

	def set_db (self, DB) :
		self.DB = DB

	def sendServerGreeting(self):
		msg = '%s' % self.IDENT
		self.sendPositiveResponse(message=msg)

	def lineReceived (self, line) : # patch things.

		self.resetTimeout()

		args = line.split(None, 2)
		rest = None
		cmd = None

		if self._pendingLiteral:
			d = self._pendingLiteral
			self._pendingLiteral = None
			d.callback(line)
			return
		elif len(args) == 3:
			tag, cmd, rest = args
		elif len(args) == 2:
			tag, cmd = args

		if not cmd :
			self.transport.loseConnection()
		else :
			cmd = cmd.upper()
			self.dispatchCommand(tag, cmd, rest)

	def dispatchCommand (self, tag, cmd, rest) :
		imap4.IMAP4Server.dispatchCommand(self, tag, cmd, rest)

		sys.stderr.write("%s %s %s\n" % (tag, cmd, rest))

	def unauth_LOGOUT(self, tag, args):
		# remove account instance
		try :
			del self.account
		except :
			pass

		self.sendUntaggedResponse('BYE Nice talking to you')
		self.sendPositiveResponse(tag, 'LOGOUT successful')
		self.transport.loseConnection()

	auth_LOGOUT = unauth_LOGOUT
	select_LOGOUT = unauth_LOGOUT
	logout_LOGOUT = unauth_LOGOUT

	def unauth_QUIT(self, tag, args):
		self.sendUntaggedResponse('BYE The correct way to terminate a session is LOGOUT')
		self.sendPositiveResponse(tag, 'QUIT successful')
		self.transport.loseConnection()

	def authenticateLogin(self, user, passwd):

		(name, domain) = message.parse_address(user)

		if not domain :
			retval = None
		else :

			# get password from DB.
			a = self.DB.fetch_one(sql.SQL_GET_USER_PASSWORD % (name, domain));

			if not a.has_key("password") :
				retval = None
			else :
				db_password = a["password"]

				# make user input passwd into sha.
				password = sha.new(passwd).hexdigest()

				if db_password != password :
					retval = None
				else :
					A = Account(user, self.DB)
					retval = A

		return retval

	def auth_STATUS(self, tag, args):
		names = imap4.parseNestedParens(args)
		if len(names) != 2:
			raise imap4.IllegalClientResponse(args)
		mailbox = self._parseMbox(names[0])

		names = names[1]
		mbox = self.account.select(mailbox, 0)

		if mbox:
			maybeDeferred(mbox.requestStatus, names).addCallbacks(
				self.__cbStatus, self.__ebStatus,
				(tag, mailbox), None, (tag, mailbox), None
			)
		else:
			self.sendNegativeResponse(tag, "Could not open mailbox")

	select_STATUS = auth_STATUS

	def __cbStatus(self, status, tag, box):
		line = ' '.join(['%s %s' % x for x in status.items()])
		self.sendUntaggedResponse("STATUS \"%s\" (%s)" % (box, line))
		self.sendPositiveResponse(tag, 'STATUS complete')

	def __ebStatus(self, failure, tag, box):
		self.sendBadResponse(tag, 'STATUS %s failed: %s' % (box, failure))

class Account (imap4.MemoryAccount) :

	mailboxes = dict()
	subscriptions = list()

	def __init__ (self, username, DB) :
		imap4.MemoryAccount.__init__(self, username)

		self.DB = DB

		# get user id
		P = Privilege.Privilege(DB, username)

		# set mailbox instance.
		#for i in self.mbox.directory.keys() :
		#	self.mailboxes[i] = True

		# set subscriptions
		self.subscriptions = self.mbox.get_all_directory_name()

		self.__set_mailboxes()

	def select (self, name, readwrite=1) :

		if self.mailboxes :
		return retval

	def addMailbox(self, name, mbox = None):

		if not self.mbox.create_directory(name) :
			raise imap4.MailboxCollision, name
		else :
			return True

	def delete(self, name):

		if not self.mbox.delete_directory(name) :
			raise MailboxException, "No such mailbox"
		else :
			return True

	def rename (self, oldname, newname) :

		ret = self.mbox.rename(oldname, newname)
		if ret == -1  :
			raise NoSuchMailbox, oldname
		elif ret == False :
			raise imap.MailboxCollision, newname
		else :
			return True

	def __set_mailboxes (self) :
		names =  self.mbox.get_all_directory_name()

		retval = list()
		for i in names :
			mbox = MailBox.MailBox(P.get_user_id(), self.DB)
			if not mbox.set_mailbox(i) :
				continue
			else :
				self.mailboxes[i] = mbox

	def listMailboxes(self, ref, wildcard):

		retval = list()

		if len(self.mailboxes.items()) < 1 : self.__set_mailboxes()

		for name, mbox in self.mailboxes.items() :
			retval.append((name, mbox))

		return retval
    

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spike@spikeekips.net>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Imap4Server.py"

