# -*- coding: utf-8 -*-
import config, sql, message, myMySQLdb

class MailBox :

	directory = {}

#	FLAGS = (
#		"ALL",			"SEEN",			"UNSEEN",
#		"ANSWERED",		"UNANSWERED",	"DELETED",
#		"UNDELETED",	"DRAFT",			"UNDRAFT",
#		"FLAGGED",		"UNFLAGGED",	"NEW",
#		"OLD",			"RECENT"
#	)

	FLAGS = ("MESSAGES", "UNSEEN", "RECENT")

	COMMAND_LIST = {
		"MESSAGES" : "getMessageCount",
		"UNSEEN" : "getUnseenCount",
		"RECENT" : "geRecentCount"
	}

	INBOX_DIRECTORY = (config.default["item"]["inbox"], config.default["item"]["tmp"])

	def __init__ (self, user_id, DB) :

		self.DB = myMySQLdb.myMySQLdb(DB)

		# set Privilege
		#self.Privilege = Privilege.Privilege(self.DB, self.username)
		#self.user_id = self.Privilege.get_user_id()
		self.user_id = user_id

		self.set_all_directory()

		# get system directory no
		self.system_directory = config.default["item"].values()

	#---------------------------------------- new method
	def set_all_directory (self) :

		self.DB.query("select no,name,parent from directory_%s" % self.user_id, True)

		self.directory_list = dict()
		self.directory_parent_list = dict()

		for i in range(self.DB.num_rows) :
			o = self.DB.fetch()

			no = int(o["no"])
			parent_no = int(o["parent"])

			self.directory_list[no] = dict()

			self.directory_list[no]["name"] = o["name"]
			self.directory_list[no]["parent"] = parent_no

			try :
				self.directory_parent_list[parent_no][o["name"]] = no
			except :
				self.directory_parent_list[parent_no] = dict()
				self.directory_parent_list[parent_no][o["name"]] = no

	def __get_valid_directory_name (self, pathspec) :

		path = pathspec.split("/")

		if config.default["item"].has_key(path[0].lower()) :
			path[0] = path[0].lower()

		return "/".join(path)

	def __get_imap_directory_name (self, pathspec) :
		path = pathspec.split("/")

		if config.default["item"].has_key(path[0].lower()) :
			path[0] = path[0].upper()

		return "/".join(path)

	def get_all_directory_name (self) :

		a = self.directory_list.copy()
		names = list()

		for i in a.keys() :
			o = self.__get_directory_full_name(i)
			names.append("/" + self.__get_imap_directory_name(o))

		return names

	def __get_directory_full_name (self, no) :

		name = self.directory_list[no]["name"]

		if self.directory_list[no]["parent"] != 0 :
			name = "%s/%s" % \
				(self.__get_directory_full_name(self.directory_list[no]["parent"]), name)

		return name

	def get_directory (self, pathspec) :

		pathspec = self.__get_valid_directory_name(pathspec)

		path = filter(None, pathspec.split('/'))

		parent = 0
		no = None
		parent_real = None
		name = None
		for i in path :
			if not self.directory_parent_list.has_key(parent) :
				parent_real, no, name = False, False, i
				break
			elif not self.directory_parent_list[parent].has_key(i) :
				parent_real, no, name = False, False, i
				break
			else :
				name = i
				no = self.directory_parent_list[parent][i]
				parent = self.directory_parent_list[parent][i]
				parent_real = self.directory_list[no]["parent"]

		retval = (no, parent_real, name)

		return retval

	def set_mailbox (self, pathspec) :

		no, parent, name = self.get_directory(pathspec)

		if not no :
			retval = False
		else :
			self.directory = no

			retval = True

		return retval

	def create_directory (self, pathspec) :

		pathspec = self.__get_valid_directory_name(pathspec)
		path = filter(None, pathspec.split('/'))

		made = False

		name = ""
		parent = 0

		for i in path :

			name = "%s/%s" % (name, i)

			no, parent_real, name_0 = self.get_directory(name)

			# if parent_real is not a proper parent.
			if parent == 0 and parent_real is False :
				break

			# if parent or no is system directory, false and continue
			elif (parent or no) in self.system_directory :
				break

			# is already registerd, continue
			elif no :
				parent = no

			# is not registered, make new
			else :
				try :
					is_registered = self.directory_parent_list[parent].has_key(i)
				except KeyError : # this directory has no sub directories.
					no = self.__create_directory(i, parent) # create new.

					if no is False :
						break
					else :
						# update directory info.
						self.directory_parent_list[parent] = dict()
						self.directory_parent_list[parent][i] = int(no)

						self.directory_list[no] = dict()

						self.directory_list[no]["name"] = i
						self.directory_list[no]["parent"] = parent

						parent = no
				else :
					if is_registered is False : # is not registered,
						no = self.__create_directory(i, parent) # create new.

						if no is False :
							made = False
							break
						else :
							# update directory info.
							self.directory_parent_list[parent][i] = int(no)

							self.directory_list[no] = dict()

							self.directory_list[no]["name"] = i
							self.directory_list[no]["parent"] = parent

							parent = no

							made = True

					else :
						continue

		return made

	def __create_directory (self, name, parent=0) :

		self.DB.start_transaction(True)

		self.DB.query( \
			"insert into directory_%s (name, parent) values('%s', '%s')" % \
			(self.user_id, myMySQLdb.escape_string(name), parent))

		if not self.DB.retval :
			retval = False
		else :
			no = self.DB.get_insert_id()
			if not no :
				retval = False
			else :
				retval = no

		self.DB.stop_transaction()

		return retval

	def delete_directory (self, pathspec) :

		pathspec = self.__get_valid_directory_name(pathspec)

		self.DB.start_transaction(True)

		# get no
		no, parent, name = self.get_directory(pathspec)

		if not no or parent == 0 :
			retval = False
		else :
			self.DB.query( \
				"delete from directory_%s where no = '%s'" % (self.user_id, no))

			retval = self.DB.retval
			self.DB.stop_transaction()

			if retval is True : del self.directory_list[no]

		# remove orphan directory
		self.__remove_orphan_directory()

		# update directory_info
		if retval is True : del self.directory_parent_list[parent][name]

		return retval

	def __remove_orphan_directory (self) :

		orphan = self.__get_orphan_directory()
		no = [i[0] for i in orphan]

		if len(no) > 0 :
			self.DB.start_transaction(True)

			add_query = " or ".join(["no = '%s'" % i for i in no ])

			self.DB.query( \
				"delete from directory_%s where %s" % (self.user_id, add_query))

			self.DB.stop_transaction()

			# update directory info.
			parents = [i[1] for i in orphan]
			names = [i[2] for i in orphan]

			for i in range(len(parents)) :
				del self.directory_parent_list[parents[i]][names[i]]


	def __get_orphan_directory (self) :
		"""
		@rtype: C{list}, [(no, parent_no, name), ...]
		"""

		a = self.directory_list.copy()

		tmp = [(i, a[i]["parent"]) for i in a.keys()]
		tmp.sort()
		couple = dict(tmp)

		orphan = list()
		for i in couple.keys() : # parent item.
			if couple[i] == 0 : continue

			if not couple.has_key(couple[i]) or couple[i] in orphan :
				orphan.append( \
					(i, self.directory_list[i]["parent"], self.directory_list[i]["name"]))

		return orphan

	def rename_directory (self, oldname, newname) :
		"""
		@rtype :
			-1, no such mailbox
			False, it's already exists.
		"""

		retval = True

		# check whether oldname is exists or not.
		no_old, parent_old, name_old = self.get_directory(oldname)
		if not no_old or parent_old == 0 :
			retval = -1

		# check whether newname is exists or not.
		if retval is True :
			no_new, parent_new, name_new = self.get_directory(newname)
			if no_new and parent_new :
				retval = False
			else :
				self.DB.start_transaction(True)

				SQL_RENAME_DIRECTORY = "update directory_%s set name = '%s' where no = '%s'"
				self.DB.query(SQL_RENAME_DIRECTORY % \
					(self.user_id, myMySQLdb.escape_string(name_new), no_old))

				retval = self.DB.retval

				self.DB.stop_transaction()

				# update directory info.
				if retval is True :
					del self.directory_parent_list[parent_old][name_old]
					self.directory_parent_list[parent_old][name_new] = no_old

					self.directory_list[no_old]["name"] = name_new

		return retval

	#--------------------------------------------------
	def getFlags (self) :

		SQL_GET_FLAG = "select flag from mail_%s where directory_num = '%s'"
		self.DB.query(SQL_GET_FLAG % (self.user_id, self.directory), True)

		flags = list()
		for i in range(self.DB.num_rows) :
			o = self.DB.fetch()
			flags.append(o["flag"])

		return flags

	def __getMessageCount(self, directory):

		o = self.DB.fetch_one(sql.SQL_GET_NUM_OF_MSG % \
			(self.user_id, directory))

		return o["nums"]

	def getRecentCount (self) :
		return 0

	def dir (self, no_list, parts, **kwargs) :

		CMD_LIST = {"FLAGS": "flag"}

		SQL_GET_ENTIREMESSAGE = "select %s from mail_%s as t1, mail_entireMessage_%s as t2 where t1.no = '%s' and t1.entireMessage = t2.no"

		# make column by parts.
		columns = list()
		for i in parts :
			if CMD_LIST.has_key(i) : columns.append("t1.%s" % CMD_LIST[i])

		add_query = ", ".join(columns)
		messages = dict()

		for i in no_list :
			o = self.DB.fetch_one(SQL_GET_ENTIREMESSAGE % \
				(add_query, self.user_id, self.user_id, i))

			if not o :
				continue
			else :
				data = dict()
				for j in parts :
					if CMD_LIST.has_key(j) : data[j] = o[CMD_LIST[j]]

				messages[i] = data

		return messages

	def getUIDValidity(self):
#		SQL_GET_NO = "select no from mail_%s where directory_num = '%s'"
#		self.DB.query(SQL_GET_NO % (self.user_id, self.directory), True)
#
#		no = list()
#		for i in range(self.DB.num_rows) :
#			o = self.DB.fetch()
#			no.append(o["flag"])
#
#		return no
		return self.directory

	def addListener (self, listner) :
		pass

	def isWriteable(self):
		if self.directory in self.system_directory :
			retval = False
		else :
			retval = True

		return retval

	def getMessageCount (self) :

		if self.directory == 1 :
			retval = 0
			for i in self.INBOX_DIRECTORY :
				retval += self.__getMessageCount(i)
		else :
			retval = self.__getMessageCount(self.directory)

		return retval

	def __getUnseenCount(self, directory):
		o = self.DB.fetch_one(sql.SQL_GET_NUM_OF_UNSEEN_MSG % \
			{"user_id" : self.user_id, \
				"directory_num" : directory})

		return o["nums"]

	def getUnseenCount(self):

		if self.directory == 1 :
			retval = 0
			for i in self.INBOX_DIRECTORY :
				retval += self.__getUnseenCount(i)
		else :
			retval = self.__getUnseenCount(self.directory)

		return retval

	def requestStatus(self, names):
		"""
		return {"MESSAGES" : number of message, "UNSEEN" : number of message}
		"""

		o = {}
		for i in names :
			method = getattr(self, self.COMMAND_LIST[i.upper()], None)
			o[i.upper()] = method()

		return o


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spike@spikeekips.net>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "MailBox.py"


