#!/bin/sh

##################################################################
# Written by Spike^ekipS <spike@spikeekips.net>
##################################################################

dialog_prog=`which dialog`

if [ ! -x "${dialog_prog}" ];then
	echo ""
	echo "= ERROR ============================================="
	echo " dialog utility must be installed in your system."
	echo " Please install dialog, or set dialog in your PATH"
	echo "====================================================="
	echo ""
	echo ""
	exit
fi

VERSION="0.6"

Dialog="${dialog_prog} --begin 5 5"

ROOT=`pwd`
tmp_file="${ROOT}/mesian_install.0"
log_file="${ROOT}/mesian_install.log"

touch ${tmp_file}
touch ${log_file}

# set ip address
__MACHINE_IP__=`hostname -i | sed -e 's/ //g'`

echo "__MACHINE_IP__=${__MACHINE_IP__}" >> ${log_file}

#---------------------------------------- message.
TITLE="Mesian ${VERSION}, Mesian is Web Mail!"

welcome_msg="\nWelcome to Mesian.\n\nWe will install the Mesian."

error_source_directory_missing="\nSource Directory, './source' does not exists.\n
Please re-unpack the mesian source package.\n\n
$ tar -jxvf mesian-x.x.tar.bz2"

enter_mesian_root_msg="\nEnter the directory to install mesian.\n\n
ex) /var/mesian"
enter_default_domain_msg="\nEnter the default domain.\n\n
ex) mesian.net"
enter_python_path_msg="\nThis python version is not valid version for
	Mesian.\n\nEnter the python path.\n\nex) /usr/bin/python2.3"
enter_mysql_path_msg="\nThis MySQL version is not valid version for
	Mesian.\n\n----------------------------------------\n
	* MySQL must be up to 4.0.x\n----------------------------------------
	\n\nEnter the correct mysql
	binary path.\n\nex) /usr/bin/mysql"
enter_db_unix_socket_msg="\nEnter the location of database socket.\n\n
ex) /var/run/mysqld/mysqld.sock"
enter_db_db_msg="\nEnter the name of database for mesian.\n\n
ex) mesian"
enter_db_user_msg="\nEnter the name of database user.\n\n
ex) mesian_user"
enter_db_password_msg="\nEnter the password of database user."
success_done="\nMesian was successfully installed.\n\nPlease read the
entire installation document in http://mesian-devel.spikeekips.net/install.php."
failed_to_install="Failed to install.\n\nCheck your environment and Retry."

goexit () {
	rm -f ./${tmp_file} &>/dev/null
	exit
}

#---------------------------------------- welcome message
${Dialog} \
	--backtitle "${TITLE}" \
	--ok-label "NEXT" \
	--msgbox "${welcome_msg}" 20 50

#---------------------------------------- check whether source directory exists or not.
if [ ! -d "./source" ];then
	${Dialog} \
		--title "ERROR" \
		--backtitle "${TITLE}" \
		--ok-label "Exit" \
		--msgbox "${error_source_directory_missing}" 20 60
	exit 1
fi

#---------------------------------------- check python version
python_prog=`which python2.3`

${python_prog} ./script/Env.py python &>/dev/null

retval=$?

__PYTHON_PATH__=`echo ${python_prog} | sed -e 's/ //g'`

while [ $retval -ne 0 ]
do
	${Dialog} \
		--title "ERROR" \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--inputbox "${enter_python_path_msg}" 20 50 "/usr/bin/python2.3" 2> $tmp_file

	if [ "$?" -eq 0 ];then
		__PYTHON_PATH__=`cat $tmp_file | sed -e 's/ //g'`
		${__PYTHON_PATH__} ./script/Env.py python
		retval=$?
	else
		goexit
	fi
done

echo "__PYTHON_PATH__=${__PYTHON_PATH__}" >> ${log_file}

#---------------------------------------- enter mesian root directory
${Dialog} \
	--backtitle "${TITLE}" \
	--ok-label "NEXT" \
	--inputbox "${enter_mesian_root_msg}" 20 50 "/var/mesian" 2> $tmp_file
__MESIAN_ROOT__=`cat $tmp_file | sed -e 's/ //g'`

if [ -z ${__MESIAN_ROOT__} ];then goexit; fi

if [ -d ${__MESIAN_ROOT__} ];then
	retval=1
else
	retval=0
fi

mkdir -p ${__MESIAN_ROOT__} &>/dev/null

if [ "$?" -ne 0 ];then
	retval=3
else
	retval=0
fi

while [ $retval -ne 0 ]
do
	if [ $retval -eq "3" ];then
		enter_mesian_root_msg2="Can not create the directory, ${__MESIAN_ROOT__}.\nCheck your environment."
	else
		enter_mesian_root_msg2="Directory ${__MESIAN_ROOT__} is alread exists.\n Enter another directory."
	fi
	${Dialog} \
		--title "ERROR" \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--inputbox "${enter_mesian_root_msg2}" 20 50 "/var/mesian2" 2> $tmp_file
	__MESIAN_ROOT__=`cat $tmp_file | sed -e 's/ //g'`

	if [ -z ${__MESIAN_ROOT__} ];then goexit; fi

	if [ -d ${__MESIAN_ROOT__} ];then
		retval=0
	else
		mkdir -p ${__MESIAN_ROOT__} &>/dev/null

		if [ "$?" -ne 0 ];then
			retval=1
		else
			retval=0
		fi
	fi

done

echo "__MESIAN_ROOT__=${__MESIAN_ROOT__}" >> ${log_file}

#---------------------------------------- enter default domain
${Dialog} \
	--backtitle "${TITLE}" \
	--ok-label "NEXT" \
	--inputbox "${enter_default_domain_msg}" 20 50 2> $tmp_file
__DEFAULT_DOMAIN__=`cat $tmp_file | sed -e 's/ //g'`

if [ -z ${__DEFAULT_DOMAIN__} ];then goexit; fi

#---------------------------------------- check mysql version
__MYSQL_PATH__="/usr/local/bin"

${python_prog} ./script/Env.py mysql ${__MYSQL_PATH__} &>/dev/null

retval=$?

while [ $retval -ne 0 ]
do
	${Dialog} \
		--title "ERROR" \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--inputbox "${enter_mysql_path_msg}" 20 50 "/usr/bin/mysql" 2> $tmp_file

	if [ "$?" -eq 0 ];then
		__MYSQL_PATH__=`cat $tmp_file | sed -e 's/ //g'`
		__MYSQL_PATH__=`dirname ${__MYSQL_PATH__}`

		${__PYTHON_PATH__} ./script/Env.py mysql ${__MYSQL_PATH__}
		retval=$?
	else
		goexit
	fi
done

echo "__MYSQL_PATH__=${__MYSQL_PATH__}" >> ${log_file}

#---------------------------------------- DATABASE setting.
retval=1
mysql_setting="We will set the MySQL
configuration.\n\n----------------------------------------\n* Must
be up to 4.0.x.\n* Must support InnoDB for Transaction.\n* Must set the
unix_socket location.\n* Must already register the user and password
for Mesian.\n----------------------------------------"
mysql_failed="Failed to set MySQL\nSet the correct value for
MySQL.\n\n----------------------------------------\n* Must be up to
4.0.x.\n* Must support InnoDB for Transaction.\n* Must set the
unix_socket location.\n* Must already register the user and password
for Mesian.\n----------------------------------------"

${Dialog} \
	--backtitle "${TITLE}" \
	--ok-label "NEXT" \
	--msgbox "${mysql_setting}" 20 50

while [ $retval -ne 0 ]
do
	if [ $retval -eq "-1" ];then
		${Dialog} \
			--backtitle "${TITLE}" \
			--ok-label "NEXT" \
			--msgbox "${mysql_failed}" 20 50
	fi

	#---------- enter db name
	${Dialog} \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--inputbox "${enter_db_db_msg}" 20 50 "mesian" 2> $tmp_file
	__DB_DB__=`cat $tmp_file | sed -e 's/ //g'`

	if [ -z ${__DB_DB__} ];then goexit; fi

	#---------- enter db user
	${Dialog} \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--inputbox "${enter_db_user_msg}" 20 50 "mesian_user" 2> $tmp_file
	__DB_USER__=`cat $tmp_file | sed -e 's/ //g'`

	if [ -z ${__DB_USER__} ];then goexit; fi

	#---------- enter db password
	${Dialog} \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--passwordbox "${enter_db_password_msg}" 20 50 2> $tmp_file
	__DB_PASSWORD__=`cat $tmp_file | sed -e 's/ //g'`

	if [ -z ${__DB_PASSWORD__} ];then goexit; fi

	#---------- enter unix_socket
	${Dialog} \
		--backtitle "${TITLE}" \
		--ok-label "NEXT" \
		--inputbox "${enter_db_unix_socket_msg}" 20 50 \
		"/var/run/mysqld/mysqld.sock" 2> $tmp_file
	__DB_UNIX_SOCKET__=`cat $tmp_file | sed -e 's/ //g'`

	if [ -z ${__DB_UNIX_SOCKET__} ];then goexit; fi

	#---------- check db configuration
	${__MYSQL_PATH__}/mysql \
		-S ${__DB_UNIX_SOCKET__} \
		-u${__DB_USER__} -p${__DB_PASSWORD__} \
		${__DB_DB__} \
		-e "select 'this is mesian!'" &>/dev/null

	if [ "$?" -ne "0" ];then
		retval=-1
	else
		retval=0
	fi
done

#---------------------------------------- view configuration
view_env="You configuration is, \n\n   MESIAN ROOT:    ${__MESIAN_ROOT__} \n   PYTHON PATH:    ${__PYTHON_PATH__} \n   DEFAULT DOMAIN: ${__DEFAULT_DOMAIN__} \n   DB DB:          ${__DB_DB__} \n   DB USER:        ${__DB_USER__} \n   DB PASSWORD:    ${__DB_PASSWORD__} \n   DB UNIX SOCKET: ${__DB_UNIX_SOCKET__}"

${Dialog} \
	--title "Is it TRUE?" \
	--backtitle "${TITLE}" \
	--ok-label "NEXT" \
	--msgbox "${view_env}" 20 50

echo "__DB_DB__=${__DB_DB__}" >> ${log_file}
echo "__DB_USER__=${__DB_USER__}" >> ${log_file}
echo "__DB_PASSWORD__=${__DB_PASSWORD__}" >> ${log_file}
echo "__DB_UNIX_SOCKET__=${__DB_UNIX_SOCKET__}" >> ${log_file}

#---------------------------------------- replace string of cfg files.
ESC__PYTHON_PATH__=`echo ${__PYTHON_PATH__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__MYSQL_PATH__=`echo ${__MYSQL_PATH__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__DEFAULT_DOMAIN__=`echo ${__DEFAULT_DOMAIN__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__MESIAN_ROOT__=`echo ${__MESIAN_ROOT__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__DB_UNIX_SOCKET__=`echo ${__DB_UNIX_SOCKET__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__DB_DB__=`echo ${__DB_DB__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__DB_USER__=`echo ${__DB_USER__} | ${__PYTHON_PATH__} ./script/escape_string.py`
ESC__DB_PASSWORD__=`echo ${__DB_PASSWORD__} | ${__PYTHON_PATH__} ./script/escape_string.py`

# replace string of cfg files.
for i in `find . -type f -regex ".*\.in$"`
do
	new_name=`echo $i | sed -e 's/\.in$//g'`

	eval "cat $i | sed -e 's/__MYSQL_PATH__/${ESC__MYSQL_PATH__}/g' -e 's/__PYTHON_PATH__/${ESC__PYTHON_PATH__}/g' -e 's/__MESIAN_ROOT__/${ESC__MESIAN_ROOT__}/g' -e 's/__DEFAULT_DOMAIN__/${ESC__DEFAULT_DOMAIN__}/g' -e 's/__DB_UNIX_SOCKET__/${ESC__DB_UNIX_SOCKET__}/g' -e 's/__DB_DB__/${ESC__DB_DB__}/g' -e 's/__DB_USER__/${ESC__DB_USER__}/g' -e 's/__DB_PASSWORD__/${ESC__DB_PASSWORD__}/g' -e 's/__MACHINE_IP__/${__MACHINE_IP__}/g' > ${new_name}"

done

#---------------------------------------- move source file to mesian root.
cd ${ROOT}
cp -r source/* ${__MESIAN_ROOT__}

# remove old file.
find ${__MESIAN_ROOT__} -type f -regex ".*\.in$" -exec rm -f {} \;

#---------------------------------------- whether qmail install or not.
choose_install_qmail_msg="We will install qmail now.\nIf you already installed qmail, just select No."

${Dialog} \
	--backtitle "${TITLE}" \
	--yesno "${choose_install_qmail_msg}" 20 50

if [ "$?" -ne 0 ];then
	cd ${ROOT}
	. ./script/set_permission.sh
	. ./script/post_install.sh

	if [ "$?" -eq 0 ];then
		#---------------------------------------- successfully done
		${Dialog} \
			--backtitle "${TITLE}" \
			--ok-label "Done" \
			--msgbox "${success_done}" 20 60

		goexit
	else
		#---------------------------------------- failed to install
		${Dialog} \
			--title "ERROR" \
			--backtitle "${TITLE}" \
			--ok-label "Done" \
			--msgbox "${failed_to_install}" 20 60

		goexit
	fi
fi

#---------------------------------------- qmail installation.

clear

echo ""
echo "=================================================="
echo " Install qmail."
echo "=================================================="
echo ""

echo "Make qmail server root directory."
mkdir -p ${__MESIAN_ROOT__}/bin &>/dev/null
mkdir -p ${__MESIAN_ROOT__}/server &>/dev/null

#---------------------------------------- check environment.
# remove old directory.
if [ -d "./compile" ];then
	cd ${ROOT}/compile
else
	echo "The directory, 'compile' does not exists. Check mesian source package."
	exit 1
fi

for i in *
do
	[ -d "$i" ] && rm -rf $i
done

#---------------------------------------- unpack the source of qmail packages.
for i in *.gz
do
	echo ">> Untar $i."
	tar zxvf "$i" &>/dev/null
done

#---------------------------------------- prepare to install.
# add qmail users & groups.
echo "Add qmail users and groups."
cd ${ROOT}
. ./script/add_qmail_user.sh &>/dev/null

#---------------------------------------- install qmail

#-------------------- daemontools
echo "Compile and Install daemontools."
cd ${ROOT}/compile/admin/daemontools-0.76

patch -p1 < ${ROOT}/compile/daemontools-0.76.errno.patch &>/dev/null
./package/compile &>/dev/null

if [ "$?" -eq 0 ];then
	cp command/* ${__MESIAN_ROOT__}/bin
else
	echo ">> Failed to install daemontools. Check your environment."
	exit 0
fi

#-------------------- ucspi-tcp.
echo "Compile and Install ucspi-tcp."
cd ${ROOT}/compile/ucspi-tcp-0.88

patch -p1 < ${ROOT}/compile/ucspi-tcp-0.88.errno.patch &>/dev/null

# edit prefix to be installed.
eval "perl -pi -e 's|\/usr\/local|${__MESIAN_ROOT__}|g' conf-home" &>/dev/null

make &>/dev/null
make setup check &>/dev/null

if [ "$?" -ne 0 ];then
	echo ">> Failed to install ucspi-tcp. Check your environment."
	exit 0
fi

#-------------------- qmail
echo "Compile and Install qmail."
cd ${ROOT}/compile/qmail-1.03

eval "perl -pi -e 's|\/var\/qmail|${__MESIAN_ROOT__}/server|g' conf-qmail" &>/dev/null

# patch
patch -p1 < ${ROOT}/compile/qmail-0.0.0.0.patch &>/dev/null
patch -p1 < ${ROOT}/compile/qmail-103.patch &>/dev/null
patch -p1 < ${ROOT}/compile/qmail-1.03.errno.patch &>/dev/null

make setup check &>/dev/null

if [ "$?" -ne 0 ];then
	echo ">> Failed to install qmail. Check your environment."
	exit 1
fi

./config-fast ${__DEFAULT_DOMAIN__} &>/dev/null

# set permission
if [ "$?" -eq 0 ];then
	cd ${ROOT}
	. ./script/set_permission.sh
else
	echo ">> Failed to set permission. Check your environment."
	exit 1
fi

#---------------------------------------- post install.
cd ${ROOT}
. ./script/post_install_qmail.sh

cd ${ROOT}
. ./script/post_install.sh

if [ "$?" -eq 0 ];then

post_install_msg="\nYou must do this job,\n\n
 Register mesian.init script as system init script.\n
$ ln -s ${__MESIAN_ROOT__}/sbin/mesian.init /etc/init.d/mesian\n
$ update-rc.d mesian defaults 95\n\n
 Start Mesian\n
$ /etc/init.d/mesian start\n"


	#---------------------------------------- successfully done
	${Dialog} \
		--backtitle "${TITLE}" \
		--ok-label "Done" \
		--msgbox "${success_done}" 20 50

	${Dialog} \
		--backtitle "${TITLE}" \
		--title "!!! IMPORTANT !!!" \
		--ok-label "FINISH" \
		--msgbox "${post_install_msg}" 20 55

	goexit
else
	#---------------------------------------- failed to install
	${Dialog} \
		--title "ERROR" \
		--backtitle "${TITLE}" \
		--ok-label "Done" \
		--msgbox "${failed_to_install}" 20 50

	goexit
fi

