import myMySQLdb, sql, message, Privilege, config

class Quota :

	"""
	Quota Policy is,
		1, 2. can not receive new message.
		3, 4. can receive new message up to surplus against domain's quota.

	column in user_priv
		quota_policy
			. set(1, 2, 3, 4)
	"""

	db = None
	address = None

	quota_size = 0
	domain_quota = 0

	current_usage =0

	surplus = 0

	policy = 1
	domain_policy = 1

	is_overquota = False

	def __init__(self, db, p) :
		"""
		p is the instance of Privilege module.
		"""

		self.db = db
		self.P = p

	def __get_quota (self, id, which="user") :
		""" """

		if which == "domain" :
			table_name = "domain_priv"
		elif which == "user" :
			table_name = "user_priv"

		DB = myMySQLdb.myMySQLdb(self.db)
		DB.query(sql.SQL_GET_QUOTA % (table_name, id), True)

		if DB.num_rows < 1 :
			retval = (None, None)
		else :
			a = DB.fetch()

			if int(a["quota"]) < 1 :
				quota_size = None
			else :
				quota_size = int(a["quota"])

			if a["quota_policy"].strip() :
				quota_policy = int(a["quota_policy"])
			else :
				quota_policy = None # default quota policy is 1.

			retval = (quota_size, quota_policy)

		return retval

	def get_quota (self) :
		""" """

		if not self.P.is_valid_user :
			pass
		else :
			b = self.__get_quota(self.P.domain_id, "domain")
			a = self.__get_quota(self.P.user_id, "user")

			# domain quota
			if not b[0] :
				self.domain_quota_size = 1
			else :
				self.domain_quota_size = b[0]

			if not b[1] :
				self.domain_policy = 1
			else :
				self.domain_policy = b[1]

			# user quota
			if not a[0] :
				self.quota_size = self.domain_quota_size
			else :
				self.quota_size = a[0]

			if not a[1] :
				self.policy = self.domain_policy
			else :
				self.policy = a[1]

			# surplus
			self.surplus = self.domain_quota_size - self.quota_size

	def get_current_usage (self) :

		DB = myMySQLdb.myMySQLdb(self.db)

		usage = 0

		# message
		DB.query(sql.SQL_GET_USAGE % self.P.user_id, True)

		if DB.num_rows > 0 :
			a = DB.fetch()
			try :
				message = int(a["sum"])
			except TypeError :
				pass
			else :
				usage += message

		# doc
		DB.query(sql.SQL_GET_USAGE_DOC % self.P.user_id, True)

		if DB.num_rows > 0 :
			a = DB.fetch()

			try :
				doc = int(a["sum"])
			except TypeError :
				pass
			else :
				usage += doc

		# card
		DB.query(sql.SQL_GET_USAGE_CARD % self.P.user_id, True)

		if DB.num_rows > 0 :
			a = DB.fetch()

			try :
				card = int(a["nums"])
			except TypeError :
				pass
			else :
				usage += card * 10000 # one card is 10kb.

		self.current_usage = usage

	def check (self, size=None) :
		"""
		Return value
			True : OK
			False : Over quota
		"""

		self.get_quota()
		self.get_current_usage()

		# is over quota?
		if self.current_usage >= self.quota_size : self.is_overquota = True

		retval = False

		# check quota size by policy.
		if self.policy in (1, 2) and not self.is_overquota :
			retval = True

		elif self.policy in (3, 4) and self.surplus > 0 :
			retval = True

		return retval

__author__ =  "Spike^ekipS <spike@spikeekips.net>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Quota.py"

