import time, os, random, sha
import sql, config, strings
import message, myMySQLdb, MailFilter, Domain
import message_db, Privilege, Quota, Send
import Log

class Deliver :

	f = None
	t = None
	entireMessage = None
	msg = None
	STATE = None

	db = None

	direction = None

	def __init__ (self, **kwargs) :
		""" kwargs["direction"] : Incoming/Outgoing """

		self.db = kwargs["db"]

		#-------------------------------------------------- set direction
		self.direction = kwargs["direction"]

	def close (self) :
		pass

	def deliver (self) :
		""" Deliver Message into in / out. """

		if self.direction == "Incoming" :
			retval = self.get()
		elif self.direction == "Outgoing" :
			retval = self.send()
		else :
			retval = False

		return retval

	#-------------------------------------------------- message handling
	def __delete_queue (self, no) :

		DB = myMySQLdb.myMySQLdb(self.db, True)

		DB.query(sql.SQL_DELETE_QUEUE % (self.direction,no))

		DB.close()

	def __delete_queue_all (self, no) :

		DB = myMySQLdb.myMySQLdb(self.db, True)

		DB.query(sql.SQL_DELETE_QUEUE_ALL % self.direction)

		DB.close()

	def __generate_message (self, entireMessage, no) :
		""" Parsing a entireMessage """

		try :
			msg = message.message(entireMessage)
		except :
			self.__delete_queue(no) # if message is not well-formed, delete it.
			retval = False
		else :
			if not msg.isMessage() : # check whether a proper message or not.
				self.__delete_queue(no)
				retval = False
			else :
				retval = msg.getAll()

		return retval

	def __run_filter (self, Privilege, address, entireMessage) :
		"""
		* return value
			-1		: remove this message
			None	: pass this message
			other	: directory number to insert

		* sequence to be applied.
			1. user filtering rule
			2. domain filtering rule
			3. global filtering rule
		"""

		# set default
		directory_num = None
		filtered = False

		F = MailFilter.Filter(self.db)
		F.set_message(entireMessage)

		#-------------------- user filtering
		F.set_name_space("user", Privilege.user_id)
		ret_filter = F.apply()

		if ret_filter != -2 :
			directory_num = ret_filter
			filtered = True

		#-------------------- domain filtering
		if not filtered and \
				Privilege.is_valid_domain and Privilege.postmaster_id \
				and not Privilege.is_escape_domain :

			F.set_name_space("domain", Privilege.postmaster_id)
			ret_filter = F.apply()

			if ret_filter != -2 :
				directory_num = ret_filter
				filtered = True

		#-------------------- global filtering
		if not filtered :
			F.set_name_space("global")
			ret_filter = F.apply()

			if ret_filter != -2 :
				directory_num = ret_filter
				filtered = True

		retval = directory_num

		return retval

	#-------------------------------------------------- get start
	def get (self) :

		DB = myMySQLdb.myMySQLdb(self.db)
		DB.query(sql.SQL_SELECT_QUEUE_ALL_NO % self.direction)

		#for i in range(0, n) :
		if DB.num_rows > 0 :
			one = True
		else :
			one = False

		retval = False

		if one == True :
			DB.query(sql.SQL_SELECT_QUEUE_ALL % self.direction, True)

			for i in range(DB.num_rows) :

				one = DB.fetch()
				if one == -1 : break

				# initiate tmp variables
				(orphan_policy, orphan_duration_time) = (None, None)

				#-------------------------------------------------- data handling
				(no, To, entireMessage, r, message_id) = \
					( int(one["no"]), one["Toaddr"], \
						one["entireMessage"], one["result"], \
						one["message_id"] \
					 )

				if not message_id :
					message_id = sha.new("%s.%s" % \
						(int(time.time()), str(random.random()))).hexdigest()

				if len(To.split("@")) < 2 :
					log = Log.Log("domain/bad")
					log.error("%s has bad To address, %s, so i removed." % (message_id, To))

					self.__delete_queue(no)
					continue

				(username, domain) = message.parse_address(To)

				log = Log.Log("domain/%s" % domain)
				log.info("%s received from db." % message_id)

				#-------------------------------------------------- Privilege check
				P = Privilege.Privilege(self.db, To)
				P.check()

				#-------------------------------------------------- Orphan message handling
				if P.is_orphan :
					DM = Domain.Domain()
					DM.set_db(DB)
					(orphan_policy, orphan_duration_time) = DM.get_orphan_policy(domain)

					if orphan_policy == "0" :  # don't receive orphan.
						self.__delete_queue(no)
						continue
					elif orphan_policy == "1" : # after duration time, remove it.
						pass
					elif orphan_policy == "2" : # do nothing.
						pass
					else :
						pass

				#------------------------------------- check if this domain is alias or not.
				if P.is_domain_alias :
					To = "%s@%s" % (username, P.domain)
					domain = P.domain

				#-------------------------------------------------- FORWARD
				# if forward was set, send a message to forwarding target.
				if P.forward :
					modified_message = message.replace_header("To", P.forward, entireMessage)

					log.info("%s will be forwarded to %s." % (message_id, P.forward))

					if modified_message :

						del entireMessage

						SS = Send.Send()
						if not SS.send(To=P.forward, \
								From="postmaster@%s" % domain, \
									message=modified_message) :
							log.error("%s failed to be forwarded to %s." % \
								(message_id, P.forward))
						else :
							log.info("%s forwarded to %s." % (message_id, P.forward))

						self.__delete_queue(no)
						log.info("%s was removed." % (message_id))

					continue

				if not P.is_valid_user :
					unKnown = True
					directory_num = None
				else :
					unKnown = False

					#-------------------------------------------------- QUOTA START
					Q = Quota.Quota(self.db, P)

					# if overquota,
					if not Q.check() :

						# rebounce message.
						M = message.message(entireMessage)
						From_all = M.get_header_field_value("Reply-To")

						if not From_all :
							From_all = M.get_header_field_value("From")

						if From_all :
							bounce_To = message.parse_address_header(From_all)[1]
							bounce_From = "postmaster@%s" % domain

							SS = Send.Send()
							try :
								SS.send(To=bounce_To, From=bounce_From, \
									message=strings.MESSAGE_OVERQUOTA % \
									{"To_address": bounce_To, "Org_address": To, "domain": domain} \
								)
							except :
								pass

						log.error( \
							"%s was bounced and removed from database becase of OVERQUOTA." % \
							message_id)
						self.__delete_queue(no)
						continue
					#-------------------------------------------------- QUOTA END

					#-------------------------------------------------- FILTER START
					filter_result = self.__run_filter(P, To, entireMessage)

					if filter_result == -1 : # remove
						log.info("%s was filtered and removed." % message_id)
						self.__delete_queue(no)
						continue
					elif filter_result == None : # default operation
						#----------------------------------------- CARD CHECK START
						C = MailFilter.AddressBook(self.db, P.user_id)
						if C.check(entireMessage) :
							directory_num = config.default["item"]["inbox"]
						#----------------------------------------- CARD CHECK END
						else :
							directory_num = config.default["item"]["default"]

					else : # filtered
						directory_num = filter_result
					#-------------------------------------------------- FILTER END

				# get message
				MSG = self.__generate_message(entireMessage, no)

				if not MSG : # if message is malformed, remove it.
					log.error("%s was malformaed message." % message_id)
					self.__delete_queue(no)
					continue

				MSG["address"] = To
				MSG["username"] = username
				MSG["domain"] = domain
				MSG["date"] = int(time.time())
				MSG["orphan"] = int(P.is_orphan)

				# insert message into DB.
				msgDB = message_db.message_db(self.db, unKnown, message = MSG)

				retval = msgDB.insert(directory_num) # insert message

				log.info("%s was successfully received to %s." % (message_id, To))
				self.__delete_queue(no) # delete message from queue

		return retval

	#-------------------------------------------------- send start
	def send (self) :

		DB = myMySQLdb.myMySQLdb(self.db)
		DB.query(sql.SQL_SELECT_QUEUE_ALL_NO % self.direction)

		#for i in range(0, n) :
		if DB.num_rows > 0 :
			one = True
		else :
			one = False

		if one == False :
			retval = False
		else :
			DB.query(sql.SQL_SELECT_QUEUE_ALL % self.direction, True)

			SS = Send.Send()

			for i in range(DB.num_rows) :

				one = DB.fetch()
				if one == -1 : break

				#-------------------------------------------------- data handling
				(no, From, To, entireMessage, m, r, message_id) = \
					(one["no"], one["Fromaddr"], one["Toaddr"], \
					one["entireMessage"], one["method"], \
					one["result"], one["message_id"])

				if not message_id :
					message_id = sha.new("%s.%s" % \
						(int(time.time()), str(random.random()))).hexdigest()

				if not message.is_valid_address(From) :
					log = Log.Log("domain/bad")
					log.error("%s has bad From address, %s, so i removed." % \
						(message_id, From))

					self.__delete_queue(no) # delete message from queue
					#From = "%s@%s" % (From, config.default["domain"])

				if not message.is_valid_address(To) :
					log = Log.Log("domain/bad")
					log.error("%s has bad To address, %s, so i removed." % \
						(message_id, To))

					self.__delete_queue(no) # delete message from queue
					#To = "%s@%s" % (To, config.default["domain"])

				(username, domain) = message.parse_address(From)

				log = Log.Log("domain/%s" % domain)
				log.info("%s will be sent." % message_id)

				# if sending a message was failed, that message will be
				# checked in result.
				if not SS.send(From=From, To=To, message=entireMessage) :

					log.error("%s was failed to send, so keep this in DB." % \
						message_id)

					DB.start_transaction(True)
					DB.query(sql.SQL_QUEUE_OUTGOING_SET_RESULT_NOTDELETED % no)
					DB.stop_transaction()

					retval = False
				else :
					log.info("%s was successfully sent to %s." % (message_id, To))
					self.__delete_queue(no) # delete message from queue

					retval = True

		return retval

__author__ =  "Spike^ekipS <spike@spikeekips.net>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Deliver.py"

